# Generated by Django 2.2.15 on 2020-11-02 19:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('verification_token', '0002'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verificationtoken',
            name='object_id',
            field=models.TextField(db_index=True),
        ),
    ]
